package arc.ex;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbManager;
import android.util.Log;
import android.widget.Toast;
import com.ftdi.j2xx.D2xxManager;
import com.ftdi.j2xx.FT_Device;



public class SC02 extends ControlModule{


	// original ///////////////////////////////
	static Context context;
	D2xxManager ftD2xx;
	FT_Device ftDev = null;
	int ftDevCount = -1;
	
    final int SC_BUF_SIZE = 256;

	/* Constructor */
	public SC02(Context parentContext )
	{
		name = "SC02";

		context = parentContext;
        txBuffer = ByteBuffer.allocate(SC_BUF_SIZE);
    	rxBuffer = ByteBuffer.allocate(SC_BUF_SIZE);
    	txBuffer.order(ByteOrder.LITTLE_ENDIAN);
       	rxBuffer.order(ByteOrder.LITTLE_ENDIAN);
       	bufferSize = SC_BUF_SIZE;
       	
		try {
    		ftD2xx = D2xxManager.getInstance( context );
    	} catch (D2xxManager.D2xxException ex) {
    		ex.printStackTrace();
    	}

    	if(!ftD2xx.setVIDPID(0x0403, 0xada1)){
    		Log.i("ftd2xx-java","setVIDPID Error");
    	}    	
                       
        IntentFilter filter = new IntentFilter();
        filter.addAction(UsbManager.ACTION_USB_DEVICE_ATTACHED);
        filter.addAction(UsbManager.ACTION_USB_DEVICE_DETACHED);
        filter.setPriority(500);
        context.registerReceiver(mUsbReceiver, filter);
        
        ftDevCount = ftD2xx.createDeviceInfoList(context);
        Toast.makeText(context, "Found FTDI device " + ftDevCount, Toast.LENGTH_LONG).show();

    }
	
    @Override
    public void transceive( int rxSize, boolean rightNow ) throws CmException
    {
    	int writtenSize, readSize, txSize;

		if (ftDev.isOpen() == false) {
			throw new CmException( this, "SC02 is not open"  );
		}
		txSize = txBuffer.position();
		txBuffer.rewind();
		
		writtenSize = ftDev.write( txBuffer.array(), txSize );
		if (writtenSize != txSize)
		{
			throw new CmException( this, String.format("Tx size missmatch: required %d  actual %d ", txSize, writtenSize) );
		}

		rxSize++; // Add SC status

		readSize = ftDev.read( rxBuffer.array(), rxSize );
		status = rxBuffer.get( readSize-1 );
		
		if( rxSize != readSize ){
			if( status != CM_NORMAL ){
				throw new CmException( 
						this, 
						String.format("Read size missmatch: required %d  actual %d  Status %d\r\n", rxSize, readSize, status ));
			}
		}
		//Log.i("DATK","sc rx " + dumpRxData( rxSize ));
		
		rxBuffer.rewind();
	}
	
    
	public void open() throws CmException
	{
		if( ftDevCount <= 0 ){
			ftDevCount = ftD2xx.createDeviceInfoList(context);
			if( ftDevCount <= 0 ){
				throw new CmException( this, "Found FTDI device " + ftDevCount );
			}
		}
		
		ftDev = ftD2xx.openByIndex(context, 0);
		
		if(ftDev == null)
		{
			throw new CmException( this, "Failed to open device by index" );
		}

		if( !ftDev.setLatencyTimer((byte) 0) ){
			throw new CmException( this, "Failed to setLatencyTimer " );
		}
		
    	totalRxSize = 0;
    	childModuleCount = 0;
    	
    	txBuffer.clear();
    	rxBuffer.clear();
    	
    	opened = true;
		
	}
	
	@Override
	public void close()
	{
		ftDevCount = -1;
		
		if( ftDev.isOpen())
		{
			ftDev.close();
		}
		opened = false;

		/*
		if(ftDev != null)
		{
			synchronized(ftDev)
			{
				if( true == ftDev.isOpen())
				{
					ftDev.close();
				}
			}
		}
		*/
		
	}

    
	public void notifyUSBDeviceDetach()
	{
		close();
	}	

    /***********USB broadcast receiver*******************************************/
    public final BroadcastReceiver mUsbReceiver = new BroadcastReceiver()  
    {
		@Override
		public void onReceive(Context context, Intent intent)
		{
			String TAG = "FragL";			
			String action = intent.getAction();
			if(UsbManager.ACTION_USB_DEVICE_DETACHED.equals(action))
			{
				Log.i(TAG,"DETACHED...");
				if( ftDev.isOpen() ) notifyUSBDeviceDetach();
			}
		}	
	};
}

